Attribute VB_Name = "modCubicSpline"
'
' ****************************************************************
' ****************************************************************
' ****************************************************************
' This program is free software: you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' version 3 as published my the Free Software Foundation:
' http://www.gnu.org/licenses/gpl.html
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' This software was originally written by Elroy Sullivan, PhD in
' cooperation with Shriners Hospitals for Children, Houston.
' Dr. Sullivan can be reached at elroysullivan@gmail.com.
' ****************************************************************
' ****************************************************************
' ****************************************************************
'
'
'*****************************************************
'*****************************************************
' Developed by Elroy Sullivan, PhD
' Contact me at elroysullivan@gmail.com
' or (832) 865-0001
'*****************************************************
'*****************************************************
'
' There are two functions in this module that are called:
'
'       CubicSplineInfoFromC3dPointData
'       PointDataFromCubicSpline
'
' The CubicSplineInfoFromC3dPointData must be called first.  This sets up
' the second derivative array that is used in the PointDataFromCubicSpline
' function.  These functions are customized to work with data that is
' read from a C3D file.  Search for PointDataFromCubicSpline for examples
' of how these functions are used.
'
Option Explicit
'
Public Type CubicSplineInfoType
    nOrig As Long
    xOrig() As Single
    yOrig() As Single
    yt() As Single
End Type
'

Public Function Cop96FromCopXY(uCopXY() As CopXY) As CopXY()
    ' This function returns a linear interpolated array from 1 to 96 (PicPelsX), with integer increments on X.
    ' NOTE:  A cubic spine was attempted, but it created un-natural spikes in a large percentage of normals, which was unacceptable.
    Dim k As Long
    Dim uCop96() As CopXY
    Dim l As Long
    Dim MinX As Single
    Dim MaxX As Single
    Dim MinY As Single
    Dim MaxY As Single
    '
    ' Create space.
    If Not CopIsDimensioned(uCopXY()) Then
        Cop96FromCopXY = uBlankCop
        Exit Function
    End If
    ReDim uCop96(1 To PicPelsX) ' ONE based, just like uCopXY.  (1 to 96)
    '
    ' Get min and max X for original COP.
    MinX = uCopXY(1).X
    MaxX = uCopXY(UBound(uCopXY)).X
    MinY = uCopXY(1).Y
    MaxY = uCopXY(UBound(uCopXY)).Y
    '
    ' Fill in X with integers.
    For l = 1 To PicPelsX
        uCop96(l).X = l
    Next l
    '
    ' Do it for Y of COP.  (Just Linear interpolation, since Cubic Spline didn't work.)
    For l = 1 To PicPelsX
        Select Case True
        Case l < MinX:  uCop96(l).Y = MinY
        Case l > MaxX:  uCop96(l).Y = MaxY
        Case Else
            For k = 2 To UBound(uCopXY)
                If uCopXY(k).X >= l Then
                    ' Linear interpolation:
                    '       (l - uCopXY(k - 1).x)               distance from first point to l.
                    '       (uCopXY(k).x - uCopXY(k - 1).x)     distance between two points capturing "l".
                    '       Above two make a ratio of how far between two points the "l" is.
                    '       (uCopXY(k).y - uCopXY(k - 1).y)     distance in "Y" between two points.
                    '       Multiply ratio by above to get interpolated delta distance.
                    '       Add above to first "Y" point to get interpolation.
                    uCop96(l).Y = uCopXY(k - 1).Y + ((uCopXY(k).Y - uCopXY(k - 1).Y) * ((l - uCopXY(k - 1).X) / (uCopXY(k).X - uCopXY(k - 1).X)))
                    Exit For
                End If
            Next k
        End Select
    Next l
    '
    ' Done.
    Cop96FromCopXY = uCop96
End Function

'Public Function Cop96FromCopXY(uCopXY() As CopXY) As CopXY()
'    ' uCop() comes in as 0 to iFrames.
'    ' This function returns a cubic splined array from 1 to 96 (PicPelsX), with integer increments on X.
'    Dim k As Long
'    Dim uCop96() As CopXY
'    Dim xOrig() As Single
'    Dim yOrig() As Single
'    Dim yt() As Single
'    Dim l As Long
'    Dim MinX As Single
'    Dim MaxX As Single
'    Dim MinY As Single
'    Dim MaxY As Single
'    '
'    ' Create space.
'    If Not CopIsDimensioned(uCopXY()) Then
'        Cop96FromCopXY = uBlankCop
'        Exit Function
'    End If
'    ReDim uCop96(1 To PicPelsX) ' ONE based, just like uCopXY.
'    ReDim xOrig(1 To UBound(uCopXY))
'    ReDim yOrig(1 To UBound(uCopXY))
'    '
'    ' Get min and max X times for original COP.
'    MinX = uCopXY(1).x
'    MaxX = uCopXY(UBound(uCopXY)).x
'    MinY = uCopXY(1).y
'    MaxY = uCopXY(UBound(uCopXY)).y
'    '
'    ' Do it for X of COP.
'    For l = 1 To PicPelsX
'        uCop96(l).x = l
'    Next l
'    '
'    ' Do it for Y of COP.
'    For k = 1 To UBound(uCopXY)
'        xOrig(k) = uCopXY(k).x
'        yOrig(k) = uCopXY(k).y
'    Next k
'    yt = CubicSpline2ndDerivatives(xOrig(), yOrig(), UBound(uCopXY))
'    '
'    For l = 1 To PicPelsX
'        Select Case True
'        Case l < MinX:  uCop96(l).y = MinY
'        Case l > MaxX:  uCop96(l).y = MaxY
'        Case Else:      uCop96(l).y = CubicSplineValue(CSng(l), xOrig(), yOrig(), UBound(uCopXY), yt())
'        End Select
'    Next l
'    '
'    ' Done.
'    Cop96FromCopXY = uCop96
'End Function

'Public Function Cop51FromCopXY(uCopXY() As CopXY, iFrames As Long) As CopXY()
'    ' uCop() comes in as 0 to iFrames.
'    ' This function returns a cubic splined 51 count array from 0 to 50.
'    Dim EndTime As Single
'    Dim k As Long
'    Dim uCop51() As CopXY
'    Dim xOrig() As Single
'    Dim yOrig() As Single
'    Dim yt() As Single
'    Dim l As Long
'    '
'    ' Create space.
'    ReDim uCop51(0 To 50)
'    ReDim xOrig(1 To iFrames)
'    ReDim yOrig(1 To iFrames)
'    '
'    ' Do it for X of COP.
'    For k = 1 To iFrames
'        xOrig(k) = (k - 1!) / (iFrames - 1!) ' We make it zero based because new Cop51 array will be zero based.
'        yOrig(k) = uCopXY(k).x
'    Next k
'    yt = CubicSpline2ndDerivatives(xOrig(), yOrig(), iFrames)
'    '
'    For l = 0 To 50
'        uCop51(l).x = CubicSplineValue(l / 50!, xOrig(), yOrig(), iFrames, yt())
'    Next l
'    '
'    ' Do it for Y of COP.
'    For k = 1 To iFrames
'        xOrig(k) = (k - 1!) / (iFrames - 1!) ' We make it zero based because new Cop51 array will be zero based.
'        yOrig(k) = uCopXY(k).y
'    Next k
'    yt = CubicSpline2ndDerivatives(xOrig(), yOrig(), iFrames)
'    '
'    For l = 0 To 50
'        uCop51(l).y = CubicSplineValue(l / 50!, xOrig(), yOrig(), iFrames, yt())
'    Next l
'    '
'    ' Done.
'    Cop51FromCopXY = uCop51
'End Function

Public Function PointDataFromCubicSpline(TheTime As Single, CubicSplineInfo As CubicSplineInfoType) As Single
    PointDataFromCubicSpline = CubicSplineValue(TheTime, CubicSplineInfo.xOrig(), CubicSplineInfo.yOrig(), CubicSplineInfo.nOrig, CubicSplineInfo.yt())
End Function

Public Function CubicSplineInfoFromC3dPointData(C3dPointData() As C3dPointFrameDataType, nOrig As Long, Dimension As Long) As CubicSplineInfoType
    '
    ' The Dimension argument specifies X=1, Y=2, or Z=3 dimensions of the 3D frame points,
    ' not to be confused with the x and y points of the cubic spline.  3D X,Y,Z points always
    ' go into the y points of the cubic spline.  The cubic spline x is always the time of the frame.
    '
    Dim xOrig() As Single
    Dim yOrig() As Single
    Dim yt() As Single
    Dim l As Long
    '
    CubicSplineInfoFromC3dPointData.nOrig = nOrig
    ReDim CubicSplineInfoFromC3dPointData.xOrig(1 To nOrig)
    ReDim CubicSplineInfoFromC3dPointData.yOrig(1 To nOrig)
    '
    ' Move the cubic spline x and y into more manageable arrays.
    For l = 1 To nOrig
        CubicSplineInfoFromC3dPointData.xOrig(l) = C3dPointData(l).Time
        Select Case Dimension
        Case 1: CubicSplineInfoFromC3dPointData.yOrig(l) = C3dPointData(l).X
        Case 2: CubicSplineInfoFromC3dPointData.yOrig(l) = C3dPointData(l).Y
        Case 3: CubicSplineInfoFromC3dPointData.yOrig(l) = C3dPointData(l).z
        End Select
    Next l
    '
    ' Get the derivative.
    CubicSplineInfoFromC3dPointData.yt = CubicSpline2ndDerivatives(CubicSplineInfoFromC3dPointData.xOrig(), CubicSplineInfoFromC3dPointData.yOrig(), nOrig)
    '
    ' Send it all back.
End Function

Private Function CubicSpline2ndDerivatives(xOrig() As Single, yOrig() As Single, nOrig As Long) As Single()
    '
    ' Given a data set consisting of a list of x values and y values,
    ' this function will smoothly interpolate a resulting output (y)
    ' value from a given input (x) value.  This procedure derives the
    ' necessary information for making calls to the CubicSplineValue procedure.
    '
    Dim i As Long
    Dim k As Long
    Dim P As Single
    Dim qn As Single
    Dim sig As Single
    Dim un As Single
    '
    ReDim u(1 To nOrig - 1) As Single
    ReDim yt(1 To nOrig) As Single ' These are the 2nd derivative values.
    '
    yt(1) = 0!
    u(1) = 0!
    '
    For i = 2 To nOrig - 1
        sig = (xOrig(i) - xOrig(i - 1)) / (xOrig(i + 1) - xOrig(i - 1))
        P = sig * yt(i - 1) + 2!
        yt(i) = (sig - 1!) / P
        u(i) = (yOrig(i + 1) - yOrig(i)) / (xOrig(i + 1) - xOrig(i)) - (yOrig(i) - yOrig(i - 1)) / (xOrig(i) - xOrig(i - 1))
        u(i) = (6! * u(i) / (xOrig(i + 1) - xOrig(i - 1)) - sig * u(i - 1)) / P
    Next i
    '
    qn = 0!
    un = 0!
    '
    yt(nOrig) = (un - qn * u(nOrig - 1)) / (qn * yt(nOrig - 1) + 1!)
    '
    For k = nOrig - 1 To 1 Step -1
        yt(k) = yt(k) * yt(k + 1) + u(k)
    Next k
    '
    CubicSpline2ndDerivatives = yt
End Function

Private Function CubicSplineValue(xNew As Single, xOrig() As Single, yOrig() As Single, nOrig As Long, yt() As Single) As Single
    ' Evaluate spline at one point.
    Dim k As Long
    Dim kLow As Long
    Dim kHigh As Long
    Dim Diff As Single
    Dim b As Single
    Dim a As Single
    Dim yNew As Single
    '
    ' Find correct interval.
    kLow = 1
    kHigh = nOrig
    Do
        k = kHigh - kLow
        If xOrig(k) > xNew Then
            kHigh = k
        Else
            kLow = k
        End If
        k = kHigh - kLow
    Loop While k > 1
    '
    Diff = xOrig(kHigh) - xOrig(kLow)
    a = (xOrig(kHigh) - xNew) / Diff
    b = (xNew - xOrig(kLow)) / Diff
    '
    yNew = a * yOrig(kLow) + b * yOrig(kHigh) + ((a * a * a - a) * yt(kLow) + (b * b * b - b) * yt(kHigh)) * (Diff * Diff) / 6!
    CubicSplineValue = yNew
End Function

