VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsGcdCalcs"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'
' ****************************************************************
' ****************************************************************
' ****************************************************************
' This program is free software: you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' version 3 as published my the Free Software Foundation:
' http://www.gnu.org/licenses/gpl.html
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' This software was originally written by Elroy Sullivan, PhD in
' cooperation with Shriners Hospitals for Children, Houston.
' Dr. Sullivan can be reached at elroysullivan@gmail.com.
' ****************************************************************
' ****************************************************************
' ****************************************************************
'
Option Explicit
'

Friend Function KinematicsExistInMotionFile() As Boolean
    If giTypeOfMotionSystem = 0 Then
        KinematicsExistInMotionFile = C3dPointLabelExists("LPelvisAngles") Or C3dPointLabelExists("RPelvisAngles")
    Else
        KinematicsExistInMotionFile = DtaGraphPlotNameExists("Trunk/Pelvis", "LPelvicTilt") Or DtaGraphPlotNameExists("Trunk/Pelvis", "RPelvicTilt")
    End If
End Function

Friend Function KineticsExistForCycle(ValidStridePointer As Long) As Boolean
    Dim StrikeOffStride As StrikeOffStrideType
    Dim SideLtr As String
    Dim GcdFrames() As GcdFrameType
    Dim GcdSingleValueMeasures As GcdSingleValueMeasuresType
    Dim Sagittal As Long
    Dim MissingCount As Long
    Dim GcdFramePtr As Long
    '
    StrikeOffStride = StrikeOffEvents.ValidStrides(ValidStridePointer)
    SideLtr = Left$(SideStr(StrikeOffStride.Side), 1)
    GcdSingleValueMeasures = GetGcdSingleValueMeasures(ValidStridePointer)
    Sagittal = GcdSingleValueMeasures.SagittalDimension
    '
    ' The "kinetics exist" check was changed so that the initial-foot-strike and terminal-foot-strike
    ' carry no extra meaning.  The determination is made strictly on the criteria that more than 10
    ' C3D frames with zero values indicates an absence of kinetics.
    '
    If C3dPointLabelExists(SideLtr & "KneeMoment") Then
        GcdFrames = GetGcdFramesForLabel(SideLtr & "KneeMoment", ValidStridePointer)
        'If Abs(GetGcdCoordinate(GcdFrames(1), Sagittal)) < 0.0001! Then
        '    KineticsExistForCycle = False
        '    Exit Function
        'End If
        'If Abs(GetGcdCoordinate(GcdFrames(GcdFrameCount), Sagittal)) < 0.0001! Then
        '    KineticsExistForCycle = False
        '    Exit Function
        'End If
        For GcdFramePtr = 1 To GcdFrameCount
            If Abs(GetGcdCoordinate(GcdFrames(GcdFramePtr), Sagittal)) < 0.0001! Then MissingCount = MissingCount + 1
        Next GcdFramePtr
        If MissingCount > 10 Then
            KineticsExistForCycle = False
            Exit Function
        End If
    Else
        KineticsExistForCycle = False
        Exit Function
    End If
    '
    KineticsExistForCycle = True
End Function

Friend Sub WriteCombinedGcdFile(frm As Form, LeftStridePointer As Long, RightStridePointer As Long, Cancelled As Boolean, WrittenFileSpec As String)
    Dim GcdFileSpec As String
    Dim StrikeOffStride As StrikeOffStrideType
    Dim SideLtr As String
    Dim SideWord As String
    Dim MsgResponse As String
    '
    ' Initial setup.
    GcdFileSpec = Left$(C3dFileSpec, Len(C3dFileSpec) - 4) & ".gcd"
    WrittenFileSpec = GcdFileSpec
    '
    ' Verify.
    If GcdFileExists(GcdFileSpec) Then
        MsgResponse = MsgBoxEx(frm.hWnd, mbOKCancel, "Writing GCD file", "The GCD file named '" & GcdFileSpec & "' already exists.  Do you wish to overwrite this file?", mbQuestion, "Yes", "Cancel")
        If MsgResponse = "Cancel" Then Cancelled = True: Exit Sub
    End If
    '
    ' Write sides.
    If LeftStridePointer > 0 Then
        StrikeOffStride = StrikeOffEvents.ValidStrides(LeftStridePointer)
        SideWord = "!" & SideStr(StrikeOffStride.Side)
        SideLtr = Left$(SideStr(StrikeOffStride.Side), 1)
        WriteGcdFileData GcdFileSpec, LeftStridePointer, SideWord, SideLtr, False, False
    End If
    If RightStridePointer > 0 Then
        StrikeOffStride = StrikeOffEvents.ValidStrides(RightStridePointer)
        SideWord = "!" & SideStr(StrikeOffStride.Side)
        SideLtr = Left$(SideStr(StrikeOffStride.Side), 1)
        If LeftStridePointer > 0 Then
            WriteGcdFileData GcdFileSpec, RightStridePointer, SideWord, SideLtr, True, True
        Else
            WriteGcdFileData GcdFileSpec, RightStridePointer, SideWord, SideLtr, False, False
        End If
    End If
End Sub

Friend Sub WriteSidedGcdFile(frm As Form, ValidStridePointer As Long, OverwriteWithoutPrompt As Boolean, Cancelled As Boolean, WrittenFileSpec As String)
    Dim StrikeOffStride As StrikeOffStrideType
    Dim SideLtr As String
    Dim SideWord As String
    Dim GcdFileSpec As String
    Dim MsgResponse As String
    '
    ' Initial setup.
    StrikeOffStride = StrikeOffEvents.ValidStrides(ValidStridePointer)
    SideWord = "!" & SideStr(StrikeOffStride.Side)
    SideLtr = Left$(SideStr(StrikeOffStride.Side), 1)
    GcdFileSpec = Left$(C3dFileSpec, Len(C3dFileSpec) - 4)
    GcdFileSpec = GcdFileSpec & "-" & SideLtr & Format$(StrikeOffStride.NumberForSide) & ".gcd"
    WrittenFileSpec = GcdFileSpec
    '
    ' Verify.
    If GcdFileExists(GcdFileSpec) And Not OverwriteWithoutPrompt Then
        MsgResponse = MsgBoxEx(frm.hWnd, mbYesNoCancel, "Writing GCD file", "The GCD file named '" & GcdFileSpec & "' already exists.  Other GCD files for other cycles or the other limb may also exist.  Do you wish to overwrite this file or all these files?", mbQuestion, "Yes", "Yes to All", "Cancel")
        If MsgResponse = "Yes to All" Then OverwriteWithoutPrompt = True
        If MsgResponse = "Cancel" Then Cancelled = True: Exit Sub
    End If
    '
    ' Write the file.
    WriteGcdFileData GcdFileSpec, ValidStridePointer, SideWord, SideLtr, False, False
End Sub

Friend Function GetGcdFramesForLabel(Label As String, ValidStridePointer As Long) As GcdFrameType()
    Dim GcdFramePtr As Long
    Dim C3dPointData() As C3dPointFrameDataType
    Dim GcdFrames(1 To GcdFrameCount) As GcdFrameType
    Dim StrideTime As Single
    Dim GcdFrameTime As Single
    Dim SplineInfoX As CubicSplineInfoType
    Dim SplineInfoY As CubicSplineInfoType
    Dim SplineInfoZ As CubicSplineInfoType
    Dim TheTime As Single
    '
    ' In VCM, when creating normalized GCD files, the C3D files seem to be offset (forward) by one C3D frame.
    ' In speculation, this may be an algorithm that insists that the frame timing be GREATER THAN the actual
    ' foot strike and foot off timing.  To be backwards compatable with plots, this "bug" is perpetuated in
    ' this program through the use of the following constant.
    Const VcmFrameCorrection = 0 ' Changed my mind.  It seems to work ok on some files.
    '
    If Not C3dPointLabelExists(Label) Then
        ' This will return empty frames.
        GetGcdFramesForLabel = GcdFrames
        Exit Function
    End If
    '
    C3dPointData = GetC3dPointDataFromLabel(Label)
    '
    SplineInfoX = CubicSplineInfoFromC3dPointData(C3dPointData(), C3dHeader.PointFrameCount, 1)
    SplineInfoY = CubicSplineInfoFromC3dPointData(C3dPointData(), C3dHeader.PointFrameCount, 2)
    SplineInfoZ = CubicSplineInfoFromC3dPointData(C3dPointData(), C3dHeader.PointFrameCount, 3)
    '
    ' Calculate the times for each of the GCD frames.
    StrideTime = StrikeOffEvents.ValidStrides(ValidStridePointer).StopStrikeTime - StrikeOffEvents.ValidStrides(ValidStridePointer).StartStrikeTime
    GcdFrameTime = StrideTime / (GcdFrameCount - 1)
    '
    For GcdFramePtr = 1 To GcdFrameCount
        ' Calculate the frame time.
        TheTime = StrikeOffEvents.ValidStrides(ValidStridePointer).StartStrikeTime + ((GcdFramePtr - 1) * GcdFrameTime) + _
                  (VcmFrameCorrection * C3dHeader.TimeBetweenFrames)
        GcdFrames(GcdFramePtr).Time = TheTime
        ' Calculate GDC points (Angles, Moments, Powers, Etc.) for this frame, in each dimension.
        GcdFrames(GcdFramePtr).X = PointDataFromCubicSpline(TheTime, SplineInfoX)
        GcdFrames(GcdFramePtr).Y = PointDataFromCubicSpline(TheTime, SplineInfoY)
        GcdFrames(GcdFramePtr).z = PointDataFromCubicSpline(TheTime, SplineInfoZ)
    Next GcdFramePtr
    '
    GetGcdFramesForLabel = GcdFrames
End Function

Friend Function GetGcdSingleValueMeasures(ValidStridePointer As Long) As GcdSingleValueMeasuresType
    Dim GcdSingleValueMeasures As GcdSingleValueMeasuresType
    Dim TheSide As FootSideEnum     ' Just used to shorten reference to side.
    Dim HaveDistanceInfo As Boolean
    Dim SplineInfoX(1 To 2) As CubicSplineInfoType ' The array is for left and right.
    Dim SplineInfoY(1 To 2) As CubicSplineInfoType ' The array is for left and right.
    Dim StartDistance As Single     ' For a particular stride.
    Dim OppositeDistance As Single  ' Point of opposite foot strike in stride event.
    Dim StopDistance As Single      ' For a particular stride.
    Dim StartDistanceY As Single    ' For a particular stride.
    Dim StopDistanceY As Single     ' For a particular stride.
    Dim StartDistanceX As Single    ' For a particular stride.
    Dim StopDistanceX As Single     ' For a particular stride.
    Dim OppositeDistanceY As Single ' Point of opposite foot strike in stride event.
    Dim OppositeDistanceX As Single ' Point of opposite foot strike in stride event.
    Dim StrideY As Single
    Dim StrideX As Single
    Dim StepY As Single
    Dim StepX As Single
    '
    GcdSingleValueMeasures.Side = StrikeOffEvents.ValidStrides(ValidStridePointer).Side
    TheSide = StrikeOffEvents.ValidStrides(ValidStridePointer).Side
    GcdSingleValueMeasures.Side = StrikeOffEvents.ValidStrides(ValidStridePointer).Side
    GcdSingleValueMeasures.StrideNumberForSide = StrikeOffEvents.ValidStrides(ValidStridePointer).NumberForSide
    HaveDistanceInfo = C3dPointLabelExists("LTOE") And C3dPointLabelExists("RTOE")
    '
    GcdSingleValueMeasures.StrideTime = StrikeOffEvents.ValidStrides(ValidStridePointer).StopStrikeTime - StrikeOffEvents.ValidStrides(ValidStridePointer).StartStrikeTime
    GcdSingleValueMeasures.Cadence = 1! / GcdSingleValueMeasures.StrideTime
    ' Puzzling, but the step time is the SECOND step in the stride, not the first.  The first step is remarked out.
    'StepTimePct = StrikeOffEvents.ValidStrides(ValidStridePointer).OppositeFootStrikeTime - StrikeOffEvents.ValidStrides(ValidStridePointer).StartStrikeTime
    GcdSingleValueMeasures.StepTimePct = (StrikeOffEvents.ValidStrides(ValidStridePointer).StopStrikeTime - StrikeOffEvents.ValidStrides(ValidStridePointer).OppositeFootStrikeTime) / GcdSingleValueMeasures.StrideTime * 100!
    GcdSingleValueMeasures.OppositeFootOffPct = (StrikeOffEvents.ValidStrides(ValidStridePointer).OppositeFootOffTime - StrikeOffEvents.ValidStrides(ValidStridePointer).StartStrikeTime) / GcdSingleValueMeasures.StrideTime * 100!
    GcdSingleValueMeasures.OppositeFootStrikePct = (StrikeOffEvents.ValidStrides(ValidStridePointer).OppositeFootStrikeTime - StrikeOffEvents.ValidStrides(ValidStridePointer).StartStrikeTime) / GcdSingleValueMeasures.StrideTime * 100!
    GcdSingleValueMeasures.SameFootOffPct = (StrikeOffEvents.ValidStrides(ValidStridePointer).SameFootOffTime - StrikeOffEvents.ValidStrides(ValidStridePointer).StartStrikeTime) / GcdSingleValueMeasures.StrideTime * 100!
    ' This is the single support percent (using entire stride time as denominator) of only the one limb being considered.
    GcdSingleValueMeasures.SingleSupportPct = (StrikeOffEvents.ValidStrides(ValidStridePointer).OppositeFootStrikeTime - StrikeOffEvents.ValidStrides(ValidStridePointer).OppositeFootOffTime) / GcdSingleValueMeasures.StrideTime * 100!
    ' This is both segments of the double support.  This should be approximately the same regardless of the limb being considered.
    GcdSingleValueMeasures.DoubleSupportPct = ((StrikeOffEvents.ValidStrides(ValidStridePointer).OppositeFootOffTime - StrikeOffEvents.ValidStrides(ValidStridePointer).StartStrikeTime) _
                                             + (StrikeOffEvents.ValidStrides(ValidStridePointer).SameFootOffTime - StrikeOffEvents.ValidStrides(ValidStridePointer).OppositeFootStrikeTime)) _
                                             / GcdSingleValueMeasures.StrideTime * 100!
    '
    If HaveDistanceInfo Then
        ' Set up for cubic spline interpolation.
        SplineInfoY(LeftSide) = CubicSplineInfoFromC3dPointData(GetC3dPointDataFromLabel("LTOE"), C3dHeader.PointFrameCount, 2)
        SplineInfoY(RightSide) = CubicSplineInfoFromC3dPointData(GetC3dPointDataFromLabel("RTOE"), C3dHeader.PointFrameCount, 2)
        SplineInfoX(LeftSide) = CubicSplineInfoFromC3dPointData(GetC3dPointDataFromLabel("LTOE"), C3dHeader.PointFrameCount, 1)
        SplineInfoX(RightSide) = CubicSplineInfoFromC3dPointData(GetC3dPointDataFromLabel("RTOE"), C3dHeader.PointFrameCount, 1)
        '
        StartDistanceY = PointDataFromCubicSpline(StrikeOffEvents.ValidStrides(ValidStridePointer).StartStrikeTime, SplineInfoY(TheSide))
        OppositeDistanceY = PointDataFromCubicSpline(StrikeOffEvents.ValidStrides(ValidStridePointer).OppositeFootStrikeTime, SplineInfoY((TheSide Mod 2) + 1))
        StopDistanceY = PointDataFromCubicSpline(StrikeOffEvents.ValidStrides(ValidStridePointer).StopStrikeTime, SplineInfoY(TheSide))
        StartDistanceX = PointDataFromCubicSpline(StrikeOffEvents.ValidStrides(ValidStridePointer).StartStrikeTime, SplineInfoX(TheSide))
        OppositeDistanceX = PointDataFromCubicSpline(StrikeOffEvents.ValidStrides(ValidStridePointer).OppositeFootStrikeTime, SplineInfoX((TheSide Mod 2) + 1))
        StopDistanceX = PointDataFromCubicSpline(StrikeOffEvents.ValidStrides(ValidStridePointer).StopStrikeTime, SplineInfoX(TheSide))
        StrideY = Abs(StopDistanceY - StartDistanceY)
        StrideX = Abs(StopDistanceX - StartDistanceX)
        StepY = Abs(OppositeDistanceY - StartDistanceY)
        StepX = Abs(OppositeDistanceX - StartDistanceX)
        ' Solve Pythagoras's Theorem.  This is the way VCM does it.
        If giTypeOfMotionSystem = 0 Then
            ' The 1.005 is a .5% correction where VCM seems to overestimate these distances.
            ' I tried other markers and I tried including the Z axis, but I could not identify the discrepancy.
            GcdSingleValueMeasures.StrideLength = Sqr(StrideX ^ 2 + StrideY ^ 2) * 1.005!
            GcdSingleValueMeasures.StepLength = Sqr(StepX ^ 2 + StepY ^ 2) * 1.005!
        Else
            GcdSingleValueMeasures.StrideLength = Sqr(StrideX ^ 2 + StrideY ^ 2)
            GcdSingleValueMeasures.StepLength = Sqr(StepX ^ 2 + StepY ^ 2)
        End If
        If StrideY > StrideX Then GcdSingleValueMeasures.SagittalDimension = 2 Else GcdSingleValueMeasures.SagittalDimension = 1
        '
        ' Speed.  This speed is in mm/sec.
        GcdSingleValueMeasures.Speed = GcdSingleValueMeasures.StrideLength / GcdSingleValueMeasures.StrideTime
    Else
        GcdSingleValueMeasures.StrideLength = 0
        GcdSingleValueMeasures.StepLength = 0
        GcdSingleValueMeasures.Speed = 0
    End If
    ' Video rate.
    GcdSingleValueMeasures.VideoRate = C3dHeader.PointFrameRate
    '
    GetGcdSingleValueMeasures = GcdSingleValueMeasures
End Function

Private Sub WriteGcdFileData(GcdFileSpec As String, ValidStridePointer As Long, SideWord As String, SideLtr As String, AppendToFile As Boolean, SkipVideoRate As Boolean)
    ' This procedure will overwrite any existing file without prompting, unless AppendToFile is specified.
    Dim GcdFileNumber As Long
    Dim GcdSingleValueMeasures As GcdSingleValueMeasuresType
    Dim Sagittal As Long
    Dim Coronal As Long
    Dim Axial As Long
    Dim GcdFramePtr As Long
    Dim GcdFrames() As GcdFrameType
    Dim TotalPower As Single
    '
    ' Open the file.
    GcdFileNumber = FreeFile
    If AppendToFile Then
        Open GcdFileSpec For Append As GcdFileNumber
    Else
        Open GcdFileSpec For Output As GcdFileNumber
        Print #GcdFileNumber, "#!DST-0.1 GCD Elroy Sullivan PhD 1.00"
    End If
    '
    ' $REFERENCE or $COMMENTS could go here.
    '
    ' Get single value measures and write into GCD file.
    GcdSingleValueMeasures = GetGcdSingleValueMeasures(ValidStridePointer)
    Print #GcdFileNumber, SideWord & "StrideTime"
    Print #GcdFileNumber, Format$(GcdSingleValueMeasures.StrideTime, "0.0###########")
    Print #GcdFileNumber, SideWord & "Cadence"
    Print #GcdFileNumber, Format$(GcdSingleValueMeasures.Cadence, "0.0###########")
    Print #GcdFileNumber, SideWord & "StePTime"
    Print #GcdFileNumber, Format$(GcdSingleValueMeasures.StepTimePct, "0.0###########")
    Print #GcdFileNumber, SideWord & "OppositeFootOff"
    Print #GcdFileNumber, Format$(GcdSingleValueMeasures.OppositeFootOffPct, "0.0###########")
    Print #GcdFileNumber, SideWord & "OppositeFootContact"
    Print #GcdFileNumber, Format$(GcdSingleValueMeasures.OppositeFootStrikePct, "0.0###########")
    Print #GcdFileNumber, SideWord & "FootOff"
    Print #GcdFileNumber, Format$(GcdSingleValueMeasures.SameFootOffPct, "0.0###########")
    Print #GcdFileNumber, SideWord & "SingleSupport"
    Print #GcdFileNumber, Format$(GcdSingleValueMeasures.SingleSupportPct, "0.0###########")
    Print #GcdFileNumber, SideWord & "DoubleSupport"
    Print #GcdFileNumber, Format$(GcdSingleValueMeasures.DoubleSupportPct, "0.0###########")
    Print #GcdFileNumber, SideWord & "StrideLength"
    Print #GcdFileNumber, Format$(GcdSingleValueMeasures.StrideLength, "0.0###########")
    Print #GcdFileNumber, SideWord & "StePLength"
    Print #GcdFileNumber, Format$(GcdSingleValueMeasures.StepLength, "0.0###########")
    Print #GcdFileNumber, SideWord & "Speed"
    Print #GcdFileNumber, Format$(GcdSingleValueMeasures.Speed, "0.0###########")
    If Not SkipVideoRate Then
        Print #GcdFileNumber, "!VideoRate"
        Print #GcdFileNumber, Format$(GcdSingleValueMeasures.VideoRate, "0.0###########")
    End If
    '
    ' Set Sagittal and Frontal dimension planes (1=x, 2=y).
    Sagittal = GcdSingleValueMeasures.SagittalDimension
    Coronal = (Sagittal Mod 2) + 1
    Axial = 3
    '
    ' Get the GCD frame data and write into the GCD file.
    '
    ' We'll start with angles.
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "WristAngles", "WristFlexExt", "WristUlRadDeviation", "", 1!, False
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "ElbowAngles", "ElbowFlexExt", "", "ElbowProSup", 1!, False
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "HeadAngles", "HeadFlexExt", "HeadLateralBend", "HeadRotation", 1!, False
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "ShoulderAngles", "ShoulderFlexExt", "ShoulderAbAdduct", "ShoulderRotation", 1!, False
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "TrunkAngles", "TrunkFlexExt", "TrunkLateralBend", "TrunkRotation", 1!, False, "SpineAngles"
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "TrunkAbsAngles", "TrunkFlexExtAbs", "TrunkLateralBendAbs", "TrunkRotationAbs", 1!, False, "ThoraxAngles"
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "PelvisAnglesRab", "PelvicTiltRab", "PelvicObliquityRab", "PelvicRotationRab", 1!, False
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "PelvisAnglesBaker", "PelvicTiltBaker", "PelvicObliquityBaker", "PelvicRotationBaker", 1!, False
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "AbsAnkleAngle", "AnkleFlexExtAbs", "", "", 1!, False
    '
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "PelvisAngles", "PelvicTilt", "PelvicObliquity", "PelvicRotation", 1!, False
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "HipAngles", "HipFlexExt", "HipAbAdduct", "HipRotation", 1!, False
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "KneeAngles", "KneeFlexExt", "KneeValgVar", "KneeRotation", 1!, False
    ' There is no Coronal ankle angle, this is done as foot progression instead.
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "AnkleAngles", "DorsiPlanFlex", "", "FootRotation", 1!, False
    ' Foot progression is only recorded in the Axial plane.
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "FootProgressAngles", "", "", "FootProgression", 1!, False
    '
    ' The following are the new foot model kinematics.
    ' The following set is labeled TIBA in the C3D.  These are TIBIA with respect to LAB, and are TibiaTilt, TibiaObliquity, & TibiaRotation.
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "TIBA", "TibiaTilt", "TibiaObliquity", "TibiaRotation", 1!, False
    ' The following set is labeled ANKA in the C3D.  These are ANKLE with respect to TIBIA, and are AnkleComplexDorsiPlanFlex, AnkleComplexValgVar, & AnkleComplexRotation.
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "ANKA", "AnkleComplexDorsiPlanFlex", "AnkleComplexValgVar", "AnkleComplexRotation", 1!, False
    ' The following set is labeled HFGA in the C3D.  These are HINDFOOT with respect to LAB, and are HindFootTilt & HindFootObliquity.
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "HFGA", "HindFootTilt", "HindFootObliquity", "", 1!, False
    ' The following set is labeled MDFA in the C3D.  These are MIDFOOT with respect to calcaneous (i.e. HFGA).  They are MidFootDorsiPlanFlex, MidFootSupPron, & MidFootAbAdduct.
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "MDFA", "MidFootDorsiPlanFlex", "MidFootSupPron", "MidFootAbAdduct", 1!, False
    ' The following set is labeled FFGA in the C3D.  These are FOREFOOT with respect to LAB.  It only uses ForeFootTilt.
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "FFGA", "ForeFootTilt", "", "", 1!, False
    ' The following set is labeled HLXA in the C3D.  These are HALLUX with respect to forefoot.  They are HalDorsiPlanFlex & HalValgVar.
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "HLXA", "HalDorsiPlanFlex", "HalValgVar", "", 1!, False
    ' The following set is labeled Supination in the C3D.  See MacWilliams write-up.  They are LeftSupination & RightSupination.
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "Supination", "LeftSupination", "RightSupination", "", 1!, False
    ' The following set is labeled Skew in the C3D.  See MacWilliams write-up.  They are LeftSkew & RightSkew.
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "Skew", "LeftSkew", "RightSkew", "", 1!, False
    '
    ' And now for moments.
    ' Strike/off reports in the params that it stores moments in the units of Newtons * millimeters.
    ' However, it appears to actually be (Newtons * millimeters) / kilograms,
    ' with the kilograms representing body mass.
    ' The VCM convention is to report moments as (Newtons * meter) / kilograms.
    ' The reduced units are actually meters^2 / seconds^2.
    ' To transform strike/off moments into VCM moments, a division by 1000 is needed
    ' to transform millimeters into meters.
    If giTypeOfMotionSystem = 0 Then
        WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "HipMoment", "HipFlexExtMoment", "HipAbAdductMoment", "HipRotationMoment", 0.001!, False
        WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "KneeMoment", "KneeFlexExtMoment", "KneeValgVarMoment", "KneeRotationMoment", 0.001!, False
        WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "AnkleMoment", "DorsiPlanFlexMoment", "FootAbAdductMoment", "FootRotationMoment", 0.001!, False
    Else
        WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "HipMoment", "HipFlexExtMoment", "HipAbAdductMoment", "HipRotationMoment", 1!, False
        WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "KneeMoment", "KneeFlexExtMoment", "KneeValgVarMoment", "KneeRotationMoment", 1!, False
        WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "AnkleMoment", "DorsiPlanFlexMoment", "FootAbAdductMoment", "FootRotationMoment", 1!, False
    End If
    '
    ' The ground reaction forces are not placed in the GCD file.
    ' GcdFrames = GetGcdFramesForLabel(SideLtr & "GroundReactionMoment", ValidStridePointer)
    '
    ' Finish up with powers.
    ' According to the params, strike/off records power as watts.  But it is actually watts / Kg,
    ' with Kg being the body mass.  The units of watts is Kg * m^3 / s^3, so, when the Kg
    ' are divided back out, strike/off power is in the units of m^3 / s^3.  These are the same
    ' units that VCM reports.
    '
    ' The overall powers at a joint are derived by solving Pythagoras's theorem in all three dimensions.
    ' Power can also go below zero, so we must multiply by the sign of the sum of power in the three dimensions.
    '
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "HipPower", "", "", "HipPower", 1!, False
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "KneePower", "", "", "KneePower", 1!, False
    WriteGcdFrameData GcdFileNumber, ValidStridePointer, SideWord, SideLtr, Sagittal, Coronal, Axial, "AnklePower", "", "", "AnklePower", 1!, False
    '
    Close GcdFileNumber
End Sub

Private Sub WriteGcdFrameData(GcdFileNumber As Long, ValidStridePointer As Long, SideWord As String, SideLtr As String, _
                              Sagittal As Long, Coronal As Long, Axial As Long, _
                              PointLabel As String, SagittalLabel As String, CoronalLabel As String, AxialLabel As String, _
                              ScaleFactor As Single, IncludeTotal As Boolean, _
                              Optional AltPointLabel As String = "")
    Dim GcdFramePtr As Long
    Dim GcdFrames() As GcdFrameType
    Dim Total As Single
    Dim TheSign As Long
    '
    Select Case True
    Case C3dPointLabelExists(SideLtr & PointLabel)
        GcdFrames = GetGcdFramesForLabel(SideLtr & PointLabel, ValidStridePointer)
        GoSub WriteToGcdFile
    Case C3dPointLabelExists(SideLtr & AltPointLabel)
        GcdFrames = GetGcdFramesForLabel(SideLtr & AltPointLabel, ValidStridePointer)
        GoSub WriteToGcdFile
    End Select
    Exit Sub
    '
WriteToGcdFile:
    If Len(SagittalLabel) <> 0 Then
        Print #GcdFileNumber, SideWord & SagittalLabel
        For GcdFramePtr = 1 To GcdFrameCount
            Print #GcdFileNumber, Format$(GetGcdCoordinate(GcdFrames(GcdFramePtr), Sagittal) * ScaleFactor, "0.0###########")
        Next GcdFramePtr
    End If
    If Len(CoronalLabel) <> 0 Then
        Print #GcdFileNumber, SideWord & CoronalLabel
        For GcdFramePtr = 1 To GcdFrameCount
            Print #GcdFileNumber, Format$(GetGcdCoordinate(GcdFrames(GcdFramePtr), Coronal) * ScaleFactor, "0.0###########")
        Next GcdFramePtr
    End If
    If Len(AxialLabel) <> 0 Then
        Print #GcdFileNumber, SideWord & AxialLabel
        For GcdFramePtr = 1 To GcdFrameCount
            Print #GcdFileNumber, Format$(GetGcdCoordinate(GcdFrames(GcdFramePtr), Axial) * ScaleFactor, "0.0###########")
        Next GcdFramePtr
    End If
    '
    Return
    '
    ' The following code is no longer used.  It has problems anyway.
    '       The sign (sense) is overly confusing and some joints (i.e., the ankle) have garbage in a particular plane.
    '
    'If IncludeTotal Then
    '    ' And now for the total.
    '    Print #GcdFileNumber, SideWord & PointLabel
    '    For GcdFramePtr = 1 To GcdFrameCount
    '        '
    '        ' Calculation of TOTAL POWER (TOTALPOWER).
    '        ' There are issues here, and there is a document titled "Calculation of Power that is Plotted.doc" that discusses them.
    '        Total = GcdFrames(GcdFramePtr).X * GcdFrames(GcdFramePtr).X + GcdFrames(GcdFramePtr).y * GcdFrames(GcdFramePtr).y + GcdFrames(GcdFramePtr).Z * GcdFrames(GcdFramePtr).Z
    '        '
    '        ' The sagittal plane should dominate, and may possibly be a better way to determine the sign.
    '        'TheSign = Sgn(GcdFrames(GcdFramePtr).X + GcdFrames(GcdFramePtr).y + GcdFrames(GcdFramePtr).Z)
    '        TheSign = Sgn(GetGcdCoordinate(GcdFrames(GcdFramePtr), Sagittal)) ' This way, only the sagittal plane's sign would be used.
    '        '
    '        Total = Sqr(Total) * TheSign
    '        Print #GcdFileNumber, Format$(Total * ScaleFactor, "0.0###########")
    '    Next GcdFramePtr
    'End If
End Sub

Private Function GetGcdCoordinate(GcdFrame As GcdFrameType, Dimension As Long) As Single
    Select Case Dimension
    Case 1: GetGcdCoordinate = GcdFrame.X
    Case 2: GetGcdCoordinate = GcdFrame.Y
    Case 3: GetGcdCoordinate = GcdFrame.z
    End Select
End Function

Private Function GcdFileExists(fle As String) As Boolean
    On Error GoTo FileExistsError
    ' If no error then something existed.
    GcdFileExists = (GetAttr(fle) And vbDirectory) = 0
    Exit Function
FileExistsError:
    GcdFileExists = False
    Exit Function
End Function

Private Function SideStr(TheSide As FootSideEnum) As String
    Select Case TheSide
    Case LeftSide
        SideStr = "Left"
    Case RightSide
        SideStr = "Right"
    Case Else
        SideStr = "None"
    End Select
End Function



