Attribute VB_Name = "modMsgBoxPlus"
'
' ****************************************************************
' ****************************************************************
' ****************************************************************
' This program is free software: you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' version 3 as published my the Free Software Foundation:
' http://www.gnu.org/licenses/gpl.html
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' This software was originally written by Elroy Sullivan, PhD in
' cooperation with Shriners Hospitals for Children, Houston.
' Dr. Sullivan can be reached at elroysullivan@gmail.com.
' ****************************************************************
' ****************************************************************
' ****************************************************************
'
'
'*****************************************************
'*****************************************************
' Developed by Elroy Sullivan, PhD
' Contact me at elroysullivan@gmail.com
' or (832) 865-0001
'*****************************************************
'*****************************************************
'
Option Explicit
Option Private Module
'
Public Enum MsgIconEnum
    mbCritical = 16
    mbExclamation = 48
    mbInformation = 64
    mbQuestion = 32
    mbSystemModal = 4096
End Enum
#If False Then ' Intellisense fix.
    Public mbCritical
    Public mbExclamation
    Public mbInformation
    Public mbQuestion
    Public mbSystemModal
#End If
'
Public Enum MsgStyleEnum
    mbYesNoCancel = &H3&
    mbYesNo = &H4&
    mbRetryCancel = &H5&
    mbOKCancel = &H1&
    mbOkOnly = &H0&
    mbAbortRetryIgnore = &H2&
End Enum
#If False Then ' Intellisense fix.
    Public mbYesNoCancel
    Public mbYesNo
    Public mbRetryCancel
    Public mbOKCancel
    Public mbOkOnly
    Public mbAbortRetryIgnore
#End If
'
Private Type MSGBOX_HOOK_PARAMS
   hWndOwner   As Long
   hHook       As Long
End Type
Private MSGHOOK As MSGBOX_HOOK_PARAMS
'
Private Declare Function GetCurrentThreadId Lib "kernel32" () As Long
Private Declare Function GetDesktopWindow Lib "user32" () As Long
Private Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long) As Long
Private Declare Function MessageBox Lib "user32" Alias "MessageBoxA" (ByVal hWnd As Long, ByVal lpText As String, ByVal lpCaption As String, ByVal wType As Long) As Long
Private Declare Function SetDlgItemText Lib "user32" Alias "SetDlgItemTextA" (ByVal hDlg As Long, ByVal nIDDlgItem As Long, ByVal lpString As String) As Long
Private Declare Function SetWindowsHookEx Lib "user32" Alias "SetWindowsHookExA" (ByVal idHook As Long, ByVal lpfn As Long, ByVal hmod As Long, ByVal dwThreadId As Long) As Long
Private Declare Function SetWindowText Lib "user32" Alias "SetWindowTextA" (ByVal hWnd As Long, ByVal lpString As String) As Long
Private Declare Function UnhookWindowsHookEx Lib "user32" (ByVal hHook As Long) As Long
Private Declare Function SetTimer Lib "user32" (ByVal hWnd As Long, ByVal nIDEvent As Long, ByVal uElapse As Long, ByVal lpTimerFunc As Long) As Long
Private Declare Function IsWindow Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function PostMessage Lib "user32" Alias "PostMessageA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long
Private Declare Function KillTimer Lib "user32" (ByVal hWnd As Long, ByVal nIDEvent As Long) As Long
'
Dim mbStyle As MsgStyleEnum
Dim mbIcon As MsgIconEnum
Dim mTitle As String
'Dim mPrompt As String
Dim But1 As String
Dim But2 As String
Dim But3 As String
'
Dim TimerID As Long
Dim TimedOut As Boolean
Dim hWndMsgBox As Long
'

Public Function MsgBoxEx(hWndOwner As Long, Style As MsgStyleEnum, Title As String, message As String, Optional Icon As MsgIconEnum, Optional ButA As String, Optional ButB As String, Optional ButC As String, _
                         Optional MilliSeconds As Long) As String
    ' This function sets your custom parameters and returns which button was pressed as a string.
    ' If a message box is a style with a "Cancel" button, the "X" on the form will be enabled.
    ' If the "X" of the form is pressed, the text of the last button (corresponding to cancel) will be returned.
    ' On an mbOkOnly textbox, the "X" will return the OK button.
    ' MilliSeconds is a timer.  If it times out, the function returns "TimedOut" string.  Can't be used with (mbAbortRetryIgnore and mbYesNo styles).
    Dim mReturn As Long
    Dim hInstance As Long
    Dim hThreadId As Long
    '
    Const WH_CBT = 5
    Const GWL_HINSTANCE = -6
    '
    Const IDOK = 1
    Const IDCANCEL = 2
    Const IDABORT = 3
    Const IDRETRY = 4
    Const IDIGNORE = 5
    Const IDYES = 6
    Const IDNO = 7
    '
    mbStyle = Style
    mbIcon = Icon
    mTitle = Title
    'mPrompt = Message
    But1 = ButA
    But2 = ButB
    But3 = ButC
    '
    hInstance = App.hInstance
    hThreadId = GetCurrentThreadId()
    MSGHOOK.hWndOwner = GetDesktopWindow()
    MSGHOOK.hHook = SetWindowsHookEx(WH_CBT, AddressOf MsgBoxHookProc, hInstance, hThreadId)
    '
    ' No default value is defined for mbAbortRetryIgnore and mbYesNo message box styles.
    ' In other words, the close (x) button is disabled, and the timer can NOT close the box.
    ' Therefore, timer can't be used.
    If Style = mbAbortRetryIgnore Or Style = mbYesNo Then MilliSeconds = 0
    If MilliSeconds <> 0 Then TimerID = SetTimer(0&, 0&, MilliSeconds, AddressOf MsgBoxTimerProc)
    '
    mReturn = MessageBox(hWndOwner, message, Space$(120), mbStyle Or mbIcon)
    '
    If TimerID <> 0 Then
        KillTimer 0&, TimerID
        TimerID = 0
    End If
    '
    If TimedOut Then
        MsgBoxEx = "TimedOut"
        TimedOut = False
    Else
        Select Case mReturn
        Case IDOK:            MsgBoxEx = But1
        Case IDABORT:         MsgBoxEx = But1
        Case IDRETRY:         MsgBoxEx = But2
        Case IDIGNORE:        MsgBoxEx = But3
        Case IDYES:           MsgBoxEx = But1
        Case IDNO:            MsgBoxEx = But2
        Case IDCANCEL ' This may be the second or third button.
            If (mbStyle And 7) = mbYesNoCancel Then
                MsgBoxEx = But3
            Else
                MsgBoxEx = But2
            End If
        End Select
    End If
End Function

Public Function MsgBoxHookProc(ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
    ' This function catches the messagebox before it opens
    ' and changes the text of the buttons - then removes the hook.
    Const IDPROMPT = &HFFFF&
    Const IDOK = 1
    Const IDCANCEL = 2
    Const IDABORT = 3
    Const IDRETRY = 4
    Const IDIGNORE = 5
    Const IDYES = 6
    Const IDNO = 7
    Const HCBT_ACTIVATE = 5
    '
    If uMsg = HCBT_ACTIVATE Then
        hWndMsgBox = wParam
        SetWindowText wParam, mTitle
        'SetDlgItemText wParam, IDPROMPT, mPrompt
        Select Case (mbStyle And 7) ' Filter out other bits.
        Case vbAbortRetryIgnore
            SetDlgItemText wParam, IDABORT, But1
            SetDlgItemText wParam, IDRETRY, But2
            SetDlgItemText wParam, IDIGNORE, But3
        Case vbYesNoCancel
            SetDlgItemText wParam, IDYES, But1
            SetDlgItemText wParam, IDNO, But2
            SetDlgItemText wParam, IDCANCEL, But3
        Case vbOKOnly
            SetDlgItemText wParam, IDOK, But1
        Case vbRetryCancel
            SetDlgItemText wParam, IDRETRY, But1
            SetDlgItemText wParam, IDCANCEL, But2
        Case vbYesNo
            SetDlgItemText wParam, IDYES, But1
            SetDlgItemText wParam, IDNO, But2
        Case vbOKCancel
            SetDlgItemText wParam, IDOK, But1
            SetDlgItemText wParam, IDCANCEL, But2
        End Select
        UnhookWindowsHookEx MSGHOOK.hHook
    End If
    MsgBoxHookProc = False
End Function

Public Function MsgBoxTimerProc(ByVal hWnd As Long, ByVal uMsg As Long, ByVal idEvent As Long, ByVal dwTime As Long) As Long
    Const WM_CLOSE = &H10
    If IsWindow(hWndMsgBox) Then
        PostMessage hWndMsgBox, WM_CLOSE, 0, ByVal 0&
        TimedOut = True
    End If
End Function

