Attribute VB_Name = "modResourceFile"
'
' ****************************************************************
' ****************************************************************
' ****************************************************************
' This program is free software: you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' version 3 as published my the Free Software Foundation:
' http://www.gnu.org/licenses/gpl.html
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' This software was originally written by Elroy Sullivan, PhD in
' cooperation with Shriners Hospitals for Children, Houston.
' Dr. Sullivan can be reached at elroysullivan@gmail.com.
' ****************************************************************
' ****************************************************************
' ****************************************************************
'
'
' The easiest way to actually create a .RES file (which gets included into the .EXE upon compilation) is as follows:
'   In the VB6 IDE, go to Add-Ins/Add-In Manager...
'   Find the Resource Manager and load it.
'   (WARNING:  Don't load on startup, as there is some bug that causes this to not work correctly.
'              Just load it anytime additional manipulation of the .RES file is desired.
'              Also note that the Resource Manager does NOT need to be loaded to get the .RES file compiled into the .EXE file.)
'   Once the Resource Manager is loaded, there will be a new icon in the toolbar (green rubics cube looking thing).
'   If you click this icon, a resource file (.RES) for the project will be started.
'   (If one was already started, it will just be opened.)
'   You can drag-and-drop any external files into this Resource Manager and they will be added to the .RES file.
'   Ideally, you will want to change their ID to a string so the following LoadResourceFileByName can be used.
'   Also, you can create groups (Resource Types) for different groups of files.
'   Also note that the resource names will not contain spaces (even if the original file names did), and they will always be UPPERCASE.
'   (This no-space and uppercase stuff is handled in the procedures below.)
'
Option Explicit
'
' Some customizations for bugs in the .RES routines.
Private Type ResFileSizesType
    FileName As String
    FileSize As Long
End Type
Dim ResFileSizes() As ResFileSizesType
'
Private Declare Function EnumResourceNames Lib "kernel32" Alias "EnumResourceNamesA" (ByVal hModule As Long, ByVal lpType As Any, ByVal lpEnumFunc As Long, ByVal lParam As Long) As Long
Private Declare Function EnumResourceTypes Lib "kernel32" Alias "EnumResourceTypesA" (ByVal hModule As Long, ByVal lpEnumFunc As Long, ByVal lParam As Long) As Long
Private Declare Function LoadLibraryEx Lib "kernel32" Alias "LoadLibraryExA" (ByVal lpLibFileName As String, ByVal hFile As Long, ByVal dwFlags As Long) As Long
Private Declare Function FreeLibrary Lib "kernel32" (ByVal hLibModule As Long) As Long
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (pTo As Any, uFrom As Any, ByVal lSize As Long)
Private Declare Function lstrlenA Lib "kernel32" (ByVal lpString As Long) As Long
Private Declare Function LoadString Lib "user32" Alias "LoadStringA" (ByVal hInstance As Long, ByVal wID As Long, ByVal lpBuffer As String, ByVal nBufferMax As Long) As Long
' LoadLibraryEx flags
Private Const DONT_RESOLVE_DLL_REFERENCES = &H1
Private Const LOAD_LIBRARY_AS_DATAFILE = &H2
Private Const LOAD_WITH_ALTERED_SEARCH_PATH = &H8
'
Dim msResDumpFolder As String
Dim mhResMakeFile As Long
'

Public Function UnpackedTemplateFileWithPath(sTemplateName As String) ' It's just the name, not the path.
    '
    ' First check to see if we've got a custom form.
    If bFileExists(CustomTemplatesFolder & sTemplateName) Then
        UnpackedTemplateFileWithPath = CustomTemplatesFolder & sTemplateName
    Else
        ' It returns the full template path for convenience.
        RetrieveResourceFileAndSaveToDisk TempFolder, sTemplateName, "Templates", True
        UnpackedTemplateFileWithPath = TempFolder & sTemplateName
    End If
End Function

Private Sub InitializeCustomSizes()
    Static bInitialized As Boolean
    '
    If bInitialized Then Exit Sub
    '
    ReDim ResFileSizes(1 To 4)
    '
    ResFileSizes(1).FileName = "EMG Report.xltx"
    ResFileSizes(1).FileSize = 73220
    '
    ResFileSizes(2).FileName = "Foot Pressure (new) (left).dotx"
    ResFileSizes(2).FileSize = 182606
    '
    ResFileSizes(3).FileName = "Foot Pressure (new) (leftright).dotx"
    ResFileSizes(3).FileSize = 185347
    '
    ResFileSizes(4).FileName = "Foot Pressure (new) (right).dotx"
    ResFileSizes(4).FileSize = 182842
    '
    bInitialized = True
End Sub

Private Function CustomSize(sFileName As String)
    ' Returns zero if no custom size found.
    Dim i As Long
    '
    InitializeCustomSizes
    CustomSize = 0
    For i = 1 To UBound(ResFileSizes)
        If sFileName = ResFileSizes(i).FileName Then
            CustomSize = ResFileSizes(i).FileSize
            Exit Function
        End If
    Next i
End Function

'
' ****************************************************************
' ****************************************************************
' The following are the generic resource file procedures.
' ****************************************************************
' ****************************************************************
'
Public Function ResourceFileSizeFromName(sResourceName As String, sResourceType As String) As Long
    Dim bb() As Byte
    '
    bb = LoadResData(Replace$(sResourceName, " ", ""), sResourceType) ' Zero based byte array.
    ResourceFileSizeFromName = UBound(bb) + 1 ' It's zero based in bb().
End Function

Public Sub RetrieveResourceFileAndSaveToDisk(sFilePath As String, sFileName As String, sResourceType As String, Optional bOverwriteIfExists As Boolean = True)
    ' Loads the specified file from the .RES (or .EXE) and then saves it to disk.
    Dim bb() As Byte
    Dim sSaveSpec As String
    '
    ' Build filespec.
    If Right$(sFilePath, 1) = "\" Then
        sSaveSpec = sFilePath & sFileName
    Else
        sSaveSpec = sFilePath & "\" & sFileName
    End If
    '
    ' Check if the file exists.  Get out if it does and we don't want to overwrite.
    ' It saves time and memory to go ahead and do this first.
    If bFileExists(sSaveSpec) Then
        If Not bOverwriteIfExists Then Exit Sub
        Kill sSaveSpec
    End If
    '
    ' Actually retrieve and save the file.
    SaveResourceFileToDisk LoadResourceFileFromName(sFileName, sResourceType), sSaveSpec, bOverwriteIfExists
End Sub

Public Function LoadResourceFileFromName(sResourceName As String, sResourceType As String) As Byte()
    ' Loads the specified data file resource from the current project's resource (.RES) file, actually the .EXE once compiled.
    '   sResourceName        Specifies the unique name of the data file resource.
    '   sResourceType        Optional. Specifies the "type" of data file being returned.  The Visual Basic default for data files is "CUSTOM".
    '
    ' Will error if not found.  Note that CASE doesn't matter in the LoadResData call.
    ' However, spaces are not allowed in resource names, so they are removed.
    LoadResourceFileFromName = LoadResData(Replace$(sResourceName, " ", ""), sResourceType) ' Returns a Byte array.
End Function

Public Function LoadResourceFileFromID(iResourceID As Integer, sResourceType As String) As Byte()
    ' Loads the specified data file resource from the current project's resource (.RES) file, actually the .EXE once compiled.
    '   iResourceID          Specifies the ID (1 to 32767) of the data file resource.
    '   sResourceType        Optional. Specifies the "type" of data file being returned.  The Visual Basic default for data files is "CUSTOM".
    '
    ' Will error if not found.  Note that CASE doesn't matter in the LoadResData call.
    LoadResourceFileFromID = LoadResData(iResourceID, sResourceType) ' Returns a Byte array.
End Function

Public Sub SaveResourceFileToDisk(DataArray() As Byte, sSaveSpec As String, Optional bOverwriteIfExists As Boolean = True)
    ' This function takes the specified data file in the form of a BYTE array and saves it out to the specified file.
    '
    ' sSaveSpec            Specifies the full path of the file to save out to.
    ' DataArray            Specifies the BYTE array that represents the data file to save out.
    ' bOverwriteIfExists   Optional. If set to TRUE and the file specified in the "sSavePath" parameter already
    '                      exists, the existing file will be overwritten with the new one.  If set to FALSE, the
    '                      existing file is left alone and the specified data file is not written out.
    '
    ' Return FALSE on error, TRUE if successful.
    Dim FileNum As Integer
    Dim sFileName As String
    Dim iCustomSize As Long
    Dim i As Long
    '
    ' Check if the file exists
    If bFileExists(sSaveSpec) Then
        If Not bOverwriteIfExists Then Exit Sub
        Kill sSaveSpec
    End If
    '
    sFileName = Mid$(sSaveSpec, InStrRev(sSaveSpec, "\") + 1)
    iCustomSize = CustomSize(sFileName)
    '
    ' Save the information to file.
    FileNum = FreeFile
    Open sSaveSpec For Binary As #FileNum
    If iCustomSize <> 0 Then
        For i = 0 To iCustomSize - 1
            Put #FileNum, , DataArray(i)
        Next i
    Else
        Put #FileNum, 1, DataArray()
    End If
    Close #FileNum
End Sub

Private Function bFileExists(fle As String) As Boolean
    On Local Error GoTo FileExistsError
    ' If no error then something existed.
    If Len(fle) <> 0 Then
        bFileExists = (GetAttr(fle) And vbDirectory) = 0
    Else
        bFileExists = False
    End If
    Exit Function
FileExistsError:
    bFileExists = False
    Exit Function
End Function

Public Sub DumpResourceFiles()
    Dim hModule As Long
    Dim FreeLib As Boolean
    Static Busy As Boolean
    Dim sModuleSpec As String
    Dim rtn As Long
    '
    If Busy Then Exit Sub
    '
    rtn = MsgBox("This will dump all resource files of PatientsAndServices.EXE into a sub-folder named 'ResourceDump'.  ALL old files in this sub-folder will be deleted." & vbCrLf & vbCrLf & _
                 "Curiously, this procedure uses the .EXE as an index, but actually dumps from the .RES file.  Any 'NEW' files in the .RES will NOT be dumped." & vbCrLf & vbCrLf & _
                 "Also, 'make.rc' and 'make.bat' files will be placed in the 'ResourceDump' folder.  These can be used to make a new PatientsAndServices.RES file.", vbOKCancel, App.Title)
    If rtn = vbCancel Then Exit Sub
    '
    '
    Busy = True
    msResDumpFolder = AppPath & "ResourceDump\"
    KillFolder msResDumpFolder
    MkDir msResDumpFolder
    '
    mhResMakeFile = FreeFile
    Open msResDumpFolder & "make.rc" For Output As mhResMakeFile
    sModuleSpec = AppPath & "PatientsAndServices.exe"
    hModule = LoadLibraryEx(sModuleSpec, 0&, LOAD_LIBRARY_AS_DATAFILE)
    Call EnumResourceTypes(hModule, AddressOf EnumResTypeProc, 0&)
    FreeLibrary hModule
    Close mhResMakeFile
    '
    mhResMakeFile = FreeFile
    Open msResDumpFolder & "make.bat" For Output As mhResMakeFile
    Print #mhResMakeFile, Chr$(34) & "C:\Program Files (x86)\Microsoft Visual Studio\VB98\Wizards\rc" & Chr$(34) & " /r /fo PatientsAndServices.res make.rc"
    Close mhResMakeFile
    Busy = False
End Sub

Private Function EnumResTypeProc(ByVal hModule As Long, ByVal lpszType As Long, ByVal lParam As Long) As Long
    If HiWord(lpszType) Then ' Custom will always be >64K and we ignore ones we didn't put there.
        MkDir msResDumpFolder & DecodeResTypeName(lpszType)
        'Debug.Print " (" & DecodeResTypeName(lpszType) & ")"
    End If
    ' Enumerate resource names of this type.
    Call EnumResourceNames(hModule, lpszType, AddressOf EnumResNameProc, lParam)
    EnumResTypeProc = True ' Continue.
End Function
   
Private Function EnumResNameProc(ByVal hModule As Long, ByVal lpszType As Long, ByVal lpszName As Long, ByVal lParam As Long) As Long
    Dim ResName As String
    Dim ResType As String
    Dim Continue As Boolean
    '
    ' Retrieve resource ID.
    ResType = DecodeResTypeName(lpszType)
    ResName = DecodeResTypeName(lpszName)
    If HiWord(lpszType) Then ' Custom will always be >64K.
        On Error Resume Next
        Err.Clear
        RetrieveResourceFileAndSaveToDisk msResDumpFolder & ResType, ResName, ResType
        If Err <> 0 Then MsgBox "There was an error with: (" & ResType & ") " & ResName, vbExclamation, App.Title
        On Error GoTo 0
        Print #mhResMakeFile, ResName & " " & ResType & " " & Chr$(34) & ToDblSlash(msResDumpFolder & ResType & "\" & ResName) & Chr$(34)
        'Debug.Print " --> "; ResType, ResName
    End If
    EnumResNameProc = True ' Continue.
End Function

Private Function ToDblSlash(sIn As String) As String
    ' Make all slashes into double-slashes.
    ' sIn is destroyed.
    Dim iStart As Long
    Dim iFound As Long
    Dim sOut As String
    '
    iStart = 1
    iFound = InStr(sIn, "\")
    Do While iFound <> 0
        sOut = sOut & Mid$(sIn, iStart, iFound - iStart + 1) & "\"
        iStart = iFound + 1
        iFound = InStr(iStart, sIn, "\")
    Loop
    sOut = sOut & Mid$(sIn, iStart)
    ToDblSlash = sOut
End Function

Private Function HiWord(LongIn As Long) As Integer
    Call CopyMemory(HiWord, ByVal (VarPtr(LongIn) + 2), 2)
End Function

Private Function DecodeResTypeName(ByVal lpszValue As Long) As String
    If HiWord(lpszValue) Then ' Pointers will always be >64K.
        DecodeResTypeName = PointerToStringA(lpszValue)
    Else
        DecodeResTypeName = "#" & CStr(lpszValue) ' Otherwise we have an ID.
    End If
End Function

Private Function PointerToStringA(lpStringA As Long) As String
    Dim Buffer() As Byte
    Dim nLen As Long
    '
    If lpStringA Then
        nLen = lstrlenA(ByVal lpStringA)
        If nLen Then
            ReDim Buffer(0 To (nLen - 1)) As Byte
            CopyMemory Buffer(0), ByVal lpStringA, nLen
            PointerToStringA = StrConv(Buffer, vbUnicode)
        End If
    End If
End Function







