Attribute VB_Name = "modWindowsFileInfo"
'
' ****************************************************************
' ****************************************************************
' ****************************************************************
' This program is free software: you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' version 3 as published my the Free Software Foundation:
' http://www.gnu.org/licenses/gpl.html
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' This software was originally written by Elroy Sullivan, PhD in
' cooperation with Shriners Hospitals for Children, Houston.
' Dr. Sullivan can be reached at elroysullivan@gmail.com.
' ****************************************************************
' ****************************************************************
' ****************************************************************
'
Option Explicit
'
Private Const MAX_PATH = 260
'
Private Type SECURITY_ATTRIBUTES
   nLength As Long
   lpSecurityDescriptor As Long
   bInheritHandle As Long
End Type
Private Type FILETIME
   dwLowDateTime As Long
   dwHighDateTime As Long
End Type
Private Type WIN32_FIND_DATA
   dwFileAttributes As Long
   ' &h01 bit = read only
   ' &h02 bit = hidden
   ' &h04 bit = system
   ' &h10 bit = folder
   ' &h20 bit = archive
   ftCreationTime As FILETIME
   ftLastAccessTime As FILETIME
   ftLastWriteTime As FILETIME
   nFileSizeHigh As Long
   nFileSizeLow As Long
   dwReserved0 As Long
   dwReserved1 As Long
   cFileName As String * MAX_PATH
   cAlternate As String * 14
End Type
Private Type SYSTEMTIME
   wYear As Integer
   wMonth As Integer
   wDayOfWeek As Integer
   wDay As Integer
   wHour As Integer
   wMinute As Integer
   wSecond As Integer
   wMilliseconds As Integer
End Type
'
Private Declare Function LocalFileTimeToFileTime Lib "kernel32" (lpLocalFileTime As FILETIME, lpFileTime As FILETIME) As Long
Private Declare Function FileTimeToLocalFileTime Lib "kernel32" (lpFileTime As FILETIME, lpLocalFileTime As FILETIME) As Long
Private Declare Function SystemTimeToFileTime Lib "kernel32" (lpSystemTime As SYSTEMTIME, lpFileTime As FILETIME) As Long
Private Declare Function FileTimeToSystemTime Lib "kernel32" (lpFileTime As FILETIME, lpSystemTime As SYSTEMTIME) As Long
Private Declare Function GetFileTime Lib "kernel32" (ByVal hFile As Long, lpCreationTime As FILETIME, lpLastAccessTime As FILETIME, lpLastWriteTime As FILETIME) As Long
Private Declare Function SetFileTime Lib "kernel32" (ByVal hFile As Long, lpCreationTime As FILETIME, lpLastAccessTime As FILETIME, lpLastWriteTime As FILETIME) As Long
Private Declare Function CloseHandle Lib "kernel32" (ByVal hObject As Long) As Long
Private Declare Function CreateFile Lib "kernel32" Alias "CreateFileA" (ByVal lpFileName As String, ByVal dwDesiredAccess As Long, ByVal dwShareMode As Long, lpSecurityAttributes As SECURITY_ATTRIBUTES, ByVal dwCreationDisposition As Long, ByVal dwFlagsAndAttributes As Long, ByVal hTemplateFile As Long) As Long
Private Declare Function FindFirstFile Lib "kernel32" Alias "FindFirstFileA" (ByVal lpFileName As String, lpFindFileData As WIN32_FIND_DATA) As Long
'
Private Const GENERIC_READ As Long = &H80000000
Private Const GENERIC_WRITE As Long = &H40000000
Private Const OPEN_EXISTING As Long = &H3
Private Const FILE_SHARE_READ As Long = &H1
Private Const FILE_SHARE_WRITE As Long = &H2
'

Public Function GetFileTimes(file_name As String, creation_date As Date, access_date As Date, modified_date As Date, local_time As Boolean) As Boolean
    ' Returns TRUE on success, FALSE on fail.
    Dim file_handle As Long
    Dim creation_filetime As FILETIME
    Dim access_filetime As FILETIME
    Dim modified_filetime As FILETIME
    Dim file_time As FILETIME
    Dim sa As SECURITY_ATTRIBUTES
    '
    ' Assume something will fail.
    GetFileTimes = False
    '
    ' Open the file.
    file_handle = CreateFile(file_name, GENERIC_READ, FILE_SHARE_READ Or FILE_SHARE_WRITE, sa, OPEN_EXISTING, 0&, 0&)
    If file_handle = 0 Then Exit Function
    '
    ' Get the times.
    If GetFileTime(file_handle, creation_filetime, access_filetime, modified_filetime) = 0 Then
        CloseHandle file_handle
        Exit Function
    End If
    '
    ' Close the file.
    If CloseHandle(file_handle) = 0 Then Exit Function
    '
    ' See if we should convert to the local
    ' file system time.
    If local_time Then
        ' Convert to local file system time.
        FileTimeToLocalFileTime creation_filetime, file_time
        creation_filetime = file_time
        '
        FileTimeToLocalFileTime access_filetime, file_time
        access_filetime = file_time
        '
        FileTimeToLocalFileTime modified_filetime, file_time
        modified_filetime = file_time
    End If
    '
    ' Convert into dates.
    creation_date = FileTimeToDate(creation_filetime)
    access_date = FileTimeToDate(access_filetime)
    modified_date = FileTimeToDate(modified_filetime)
    '
    GetFileTimes = True ' We succeeded.
End Function

Public Function SetFileTimes(file_name As String, creation_date As Date, access_date As Date, modified_date As Date, local_times As Boolean) As Boolean
    ' Returns TRUE on success, FALSE on fail.
    Dim file_handle As Long
    Dim creation_filetime As FILETIME
    Dim access_filetime As FILETIME
    Dim modified_filetime As FILETIME
    Dim file_time As FILETIME
    Dim sa As SECURITY_ATTRIBUTES
    '
    ' Assume something will fail.
    SetFileTimes = False
    '
    ' Convert the dates into FILETIMEs.
    creation_filetime = DateToFileTime(creation_date)
    access_filetime = DateToFileTime(access_date)
    modified_filetime = DateToFileTime(modified_date)
    '
    ' Convert the file times into system file times.
    If local_times Then
        LocalFileTimeToFileTime creation_filetime, file_time
        creation_filetime = file_time
        '
        LocalFileTimeToFileTime access_filetime, file_time
        access_filetime = file_time
        '
        LocalFileTimeToFileTime modified_filetime, file_time
        modified_filetime = file_time
    End If
    '
    ' Open the file.
    file_handle = CreateFile(file_name, GENERIC_WRITE, FILE_SHARE_READ Or FILE_SHARE_WRITE, sa, OPEN_EXISTING, 0&, 0&)
    If file_handle = 0 Then Exit Function
    '
    ' Set the times.
    If SetFileTime(file_handle, creation_filetime, access_filetime, modified_filetime) = 0 Then
        CloseHandle file_handle
        Exit Function
    End If
    '
    ' Close the file.
    If CloseHandle(file_handle) = 0 Then Exit Function
    '
    SetFileTimes = True ' We succeeded.
End Function

Public Function FileSize(sFileSpec As String) As Long
    ' This will not work correctly for files over 2 gig.
    ' Also, if the file doesn't exist, it returns zero.
    Dim wfd As WIN32_FIND_DATA
    Dim h As Long
    Const INVALID_HANDLE_VALUE = -1
    '
    h = FindFirstFile(sFileSpec, wfd)
    If h = INVALID_HANDLE_VALUE Then Exit Function
    '
    FileSize = wfd.nFileSizeLow
End Function

Public Function FileCreatedDate(sFileSpec As String) As Date
    ' If the file doesn't exist, it returns zero (12/30/1899 midnight).
    Dim wfd As WIN32_FIND_DATA
    Dim h As Long
    Const INVALID_HANDLE_VALUE = -1
    '
    h = FindFirstFile(sFileSpec, wfd)
    If h = INVALID_HANDLE_VALUE Then Exit Function
    '
    FileCreatedDate = dFileTimeToDouble(wfd.ftCreationTime, True)
End Function

Public Function FileModifiedDate(sFileSpec As String) As Date
    ' If the file doesn't exist, it returns zero (12/30/1899 midnight).
    Dim wfd As WIN32_FIND_DATA
    Dim h As Long
    Const INVALID_HANDLE_VALUE = -1
    '
    h = FindFirstFile(sFileSpec, wfd)
    If h = INVALID_HANDLE_VALUE Then Exit Function
    '
    FileModifiedDate = dFileTimeToDouble(wfd.ftLastWriteTime, True)
End Function

Public Function FileAccessedDate(sFileSpec As String) As Date
    ' If the file doesn't exist, it returns zero (12/30/1899 midnight).
    Dim wfd As WIN32_FIND_DATA
    Dim h As Long
    Const INVALID_HANDLE_VALUE = -1
    '
    h = FindFirstFile(sFileSpec, wfd)
    If h = INVALID_HANDLE_VALUE Then Exit Function
    '
    FileAccessedDate = dFileTimeToDouble(wfd.ftLastAccessTime, True)
End Function

Private Function dFileTimeToDouble(ftUTC As FILETIME, Optional Localize As Boolean = False) As Double
   Dim FT As FILETIME
   Dim st As SYSTEMTIME
   Dim d As Double
   Dim t As Double
   ' Convert to local filetime, if necessary.
   If Localize Then
      FileTimeToLocalFileTime ftUTC, FT
   Else
      FT = ftUTC
   End If
   ' Convert to system time structure.
   FileTimeToSystemTime FT, st
   ' Convert to VB-style date (double).
   dFileTimeToDouble = DateSerial(st.wYear, st.wMonth, st.wDay) + TimeSerial(st.wHour, st.wMinute, st.wSecond)
End Function

Private Function DateToFileTime(DateTime As Date) As FILETIME
    Dim stLocal As SYSTEMTIME
    Dim stUniversal As SYSTEMTIME
    Dim ftResult As FILETIME
    '
    stLocal.wYear = Year(DateTime)
    stLocal.wMonth = Month(DateTime)
    stLocal.wDay = Day(DateTime)
    stLocal.wHour = Hour(DateTime)
    stLocal.wMinute = Minute(DateTime)
    stLocal.wSecond = Second(DateTime)
    '
    SystemTimeToFileTime stLocal, ftResult
    DateToFileTime = ftResult
End Function

Private Function FileTimeToDate(FT As FILETIME) As Date
    Dim stLocal As SYSTEMTIME
    Const ZERO_DATE As Date = 0!
    '
    If (FT.dwHighDateTime <> 0) And (FT.dwLowDateTime <> 0) Then
        FileTimeToSystemTime FT, stLocal
        FileTimeToDate = DateSerial(stLocal.wYear, stLocal.wMonth, stLocal.wDay) + TimeSerial(stLocal.wHour, stLocal.wMinute, stLocal.wSecond)
    Else
        FileTimeToDate = ZERO_DATE
    End If
End Function

